#The purpose of this script is to change untrustworthy calls to "NA"
#untrustworthy calls can be caused by sequencing errors or mismapping of tags
#script first id's recombination breakpoints, then determines if the call is likely to be an error
#if the next three calls on different tags match the breakpoint call, it is left as is
#if the next three calls do not match, the breakpoint call is changed to "NA"
#Format issues: unlike the RIL version of this script, the data should already be in R/Qtl format for use with this script,
#except columns with containing chrom and pos should have header names "chrom" and "pos", first column should contain the name of the SNP under header name "rs#"
#and all the rest of the cols have individuals
#This script is intended for use with F2 populations that are genotyped with either dominant or codominant markers


import argparse
import pandas
import sys
import numpy as np

def test_break(series, start, last, ct):
    
    book= {'A':('B','H','C'), 'B':('A','H','D'), 'H':('A','B'),
           'C':('A'), 'D':('B')}
    
    orig_val= series[start]
    
    if orig_val not in book[last]:
        return True

    count=-1
    true_breakpoints= book[orig_val]
    
    for idx, val in series.ix[start:].iteritems():
 
        if (val != orig_val and pandas.notnull(val)):
            if val in true_breakpoints:
                return False
        count+= 1
                
        if count >= ct:
            return True
    return False
        
    

def error_correction(hmp, e, ct, o):
    
    bad_report=[]
    mod=[]
  
    c_grouped= hmp.groupby('chrom')

    for chrom, hmp in c_grouped:
        corrections=0

        for col_idx, col in hmp.ix[:,3:].iteritems():
            last= None
            for row_idx, val in col.iteritems():
                if pandas.isnull(val):
                    continue
                            
                is_good_break= (last != None and val != last and test_break(col, row_idx, last, ct))

                if is_good_break or not last:
                    last= val

                elif val != last:

                    hmp.ix[row_idx, col_idx]= np.nan
                    corrections += 1
                    
                    info= [hmp.ix[row_idx, 'rs#'],hmp.ix[row_idx, 'pos'],col_idx]
                    bad_report.append(info)
            
        print("changed "+str(corrections)+ " calls on chromosome "+str(chrom)+" to NA")

        mod.append(hmp)

    joint= pandas.concat(mod)
    joint= joint.sort_index(by=['chrom','pos'])
    joint= joint.reindex(columns=["rs#","chrom","pos"]+ sorted(joint.columns[3:], key= int))

    if e!= None:
        err= pandas.DataFrame(bad_report)
        err= err.rename(columns={0: 'rs#', 1:'pos', 2:'individual'})
        err.to_csv(e, sep= '\t', index=False)
        
    joint.to_csv(o, index=False, na_rep="NA")
    

def main():

    parser= argparse.ArgumentParser(description= "PLUMAGE v. 2.0: pipeline for GBS error correction")

    parser.add_argument('-i', metavar= 'infile', type=argparse.FileType('r'), nargs='?', default=sys.stdin,
                        help= 'path/file containing a .csv file in R/qtl format for sequence error correction default = stdin')

    parser.add_argument('-ct',metavar="breakpoint cutoff", type= int, nargs='?', default=5,
                        help= "number of matching calls that must follow a breakpoint for it to be considered 'good', defaults to 5")
    parser.add_argument('-o', nargs= '?', metavar= 'outfile', type=argparse.FileType('w'),
                        default= sys.stdout, help= "file path/name if you want to write results to a file\n"
                        "default = stdout")
    parser.add_argument('-e', nargs='?', metavar= 'error_report', default= None,
                        help= "specify a file path/name if you want to save a report of sequencing errors")
    args = parser.parse_args()

    hmp= pandas.read_table(args.i, sep=",")
    error_correction(hmp, args.e, args.ct, args.o)
    
    
if __name__ == "__main__":
  main()    
    
        
